#!/usr/local/bin/python
#    Copyright 2007 Stuart Stevenson
#    Derived from a work by Chris Radek which was
#    derived from a work by John Kasunich and Jeff Epler
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from vismach import *
import hal
import math
import sys

# give endpoint Z values and radii
# resulting cylinder is on the Z axis
class HalToolCylinder(CylinderZ):
    def __init__(self, comp, *args):
	CylinderZ.__init__(self, *args)
	self.comp = comp

    def coords(self):
        return -self.comp.tool_length, .75, 0, .75

c = hal.component("cincigui")
c.newpin("joint0", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("joint1", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("joint2", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("joint3", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("joint4", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("joint5", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("joint6", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("joint7", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("joint8", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("pivot_length", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("tool_length", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("tool_dia", hal.HAL_FLOAT, hal.HAL_IN)
c.ready()

pivot_len=5.25
tool_radius=1.0

for setting in sys.argv[1:]: exec setting

#########################################################################

tooltip = Capture()
tool = Collection([HalTranslate([tooltip], c, "tool_length", 0,0,-1),
                   HalToolCylinder(c),
                   CylinderZ(0, 4.0, 1.0, 4.0)
                  ,CylinderZ(.75, 5.0, 2.25, 5.0)
                  ,Box(-7.5,-7.5,2.0, 7.5,7.5,37.5)
                   ])

tool = Translate([tool], 0, 0, -pivot_len)

tool = Collection([tool,
                   CylinderY(4.0,3.0, .4,3.0)
                  ,CylinderY(-2.0,2.375,2.0,2.375)
                   ])

tool = HalRotate([tool],c,"joint3",1,1,0,0)

#########################################################################

wrist = Collection([tool,
                    Box(-15.75,-7.875,-4.0, -7.875,7.875,35.50)
                    ,Box(7.875,-7.875,-4.0, 4.0,7.875,35.5)
                    ])

wrist = HalRotate([wrist],c,"joint4",1,0,1,0)

#########################################################################

ram = Collection([wrist,
                  Box(-15.75,7.875,-4.0, 15.75,17.75,59.0)
                  ])

ram = Translate([ram], 0,0,0)

ram = HalTranslate([ram],c,"joint2",0,0,1)

ram = Collection([ram,
                  Box(-25.5,21.625,15.75, 25.5,157.5,78.75)
                 ,Box(-11.75,17.75,11.75, 11.75,21.75,98.5)
                  ])
    
ram = HalTranslate([ram],c,"joint1",0,1,0)

#########################################################################

Xtable = Capture()

table = Collection([Xtable,
                  Box(-60,-19.75,-59.0,  60,19.75,-67.0)
                  ])

table = HalTranslate([table],c,"joint0",-1,0,0)

#########################################################################

work = Capture()

base = Collection([work,
                  Box(-29.5,21.625,-108.25, 29.5,49.25,-72.875)
     	         ,Box(-29.5,49.25,-108.25, 29.5,128,15.75)
                 ,Box(137.75,21.625,-68.875, -137.75,-21.625,-108.25)
        ])

#########################################################################

model = Collection([ram, table, base])

#########################################################################

main(model, tooltip, work, 400)
